// $Id: editor_plugin.js,v 1.1.2.3 2010/05/06 21:06:03 franz Exp $
(function() {
  // Load plugin specific language pack.
  tinymce.PluginManager.requireLangPack('media_library');

  tinymce.create('tinymce.plugins.MediaLibraryPlugin', {
    /**
     * Initialize the plugin, executed after the plugin has been created.
     *
     * This call is done before the editor instance has finished it's
     * initialization so use the onInit event of the editor instance to
     * intercept that event.
     *
     * @param ed
     *   The tinymce.Editor instance the plugin is initialized in.
     * @param url
     *   The absolute URL of the plugin location.
     */
    init : function(ed, url) {
      // Register the MediaLibrary execCommand.
      ed.addCommand('MediaLibrary', function() {
        // captionTitle and captionDesc for backwards compatibility.
        // TODO: Check this: Lots of useless information
        // var data = {nid: '', title: '', captionTitle: '', desc: '', captionDesc: '', link: '', url: '', align: '', width: '', height: '', id: ed.id, action: 'insert', lightbox: ''};
        var data = {action: 'insert', id: ed.id};
        
        // Recursevily goes back in the DOM tree to search for container
        var node = ed.selection.getNode();
        while (node.parentNode && node.className != 'mceItemMediaLibrary')
          node = node.parentNode;

        if (node.className == 'mceItemMediaLibrary') {
          ed.selection.select(node);
          // Get data from object. TODO: get additional attributes like these? how?
          //data.width = node.width;
          //data.height = node.height;
          //data.align = node.align;
          // Expand inline tag in alt attribute
          var tag_el = node.getElementsByClassName('ml-tag');
          var chunks = tag_el[0].innerHTML.split('|');
          chunks[0].replace(/([^:]+):([^\]]*)/,  function(o, property, value) {
            data.type = value;
          });
          for (var i in chunks) {
            chunks[i].replace(/([^=]+)=([^\]]*)/g, function(o, property, value) {
              data[property] = value;
            });
          }
          data.action = 'update';
        }
        var object = $(this);
        $(this).addClass('ctools-ajaxing');
        Drupal.CTools.Modal.show();
        data.tiny = true;
        try {
          url = 'index.php?q=media-library/main';
          $.ajax({
            type: "POST",
            url: url,
            data: data,
            global: true,
            success: Drupal.CTools.AJAX.respond,
            error: function() {
              alert("An error occurred while attempting to process " + url);
            },
            complete: function() {
              object.removeClass('ctools-ajaxing');
            },
            dataType: 'json'
          });
        }
        catch (err) {
          alert("An error occurred while attempting to process " + url);
          $(this).removeClass('ctools-ajaxing');
          return false;
        }
      });

      // Register Media Library  button.
      ed.addButton('media_library', {
        title : 'media_library.desc',
        cmd : 'MediaLibrary',
        // TODO: change the image.
        image : url + '/images/drupalimage.gif'
      });

      // Load Media Library's CSS for editor contents on startup.
      // TODO: Check this
      ed.onInit.add(function() {
        if (!ed.settings.drupalimage_skip_plugin_css) {
          ed.dom.loadCSS(url + "/css/media_library.css");
        }
      });

      // Replace images with inline tags in editor contents upon data.save.
      // @todo Escape regular | pipes.
      ed.onBeforeGetContent.add(function(ed, data) {
        if (!data.save) {
          return;
        }
        jQuery.each(ed.dom.select('.mceItemMediaLibrary', data.content), function(node) {
          var tag = this.getElementsByClassName('ml-tag')[0].innerHTML;
          ed.dom.setOuterHTML(this, tag);
        });
      });

      // Replace inline tags in data.content with images.
      ed.onBeforeSetContent.add(function(ed, data) {
        if ( typeof placeholder  == 'undefined' ) {
          placeholder = 0;
        }
        data.content = data.content.replace(/\[media:([^\|]*\|[^\[\]]+)\]/g, function(orig, match) {
          var node = {}, chunks = match.split('|');
          node.type = chunks[0];
          for (var i in chunks) {
            chunks[i].replace(/([^=]+)=(.*)/g, function(o, property, value) {
              node[property] = value;
            });
          }
          node.name = 'mceItemMediaLibrary';
          node.tag = '[media:' + match + ']';
          node.placeholder = placeholder;
          try {
            url = 'index.php?q=media-library/preview/' + node.type ;
            $.ajax({
              type: "POST",
              url: url,
              data: node,
              global: true,
              success: function(data) {
                content = Drupal.theme('media_library_preview', data, 0);
                ed.dom.setOuterHTML(ed.dom.select('#media-library-ajaxing-' + node.placeholder)[0], content);
                //ed.execCommand('mceInsertContent', false, content);
              },
              error: function() {
                alert("An error occurred while attempting to process " + url);
              },
              complete: function(data) {
                //object.removeClass('ctools-ajaxing');
              },
              dataType: 'json'
            });
          }
          catch (err) {
            alert("An error occurred while attempting to process " + url);
            //$(this).removeClass('ctools-ajaxing');
            return false;
          }
          return ed.dom.createHTML('div', {class:"ctools-ajaxing", id: "media-library-ajaxing-" + placeholder++}, Drupal.t('Loading...'));
          /*node.src = Drupal.settings.basePath + 'index.php?q=imagefield_assist/preview/' + node.fid + '/' + node.preset;
	  if (this.lightbox != '') light = '|lightbox=true';
	  else light = '';
          node.alt = 'fid=' + node.fid + '|preset=' + node.preset + light + '|title=' + node.title + '|desc=' + node.desc;
          if (node.link.indexOf(',') != -1) {
            var link = node.link.split(',', 2);
            node.alt += '|link=' + link[0] + '|url=' + link[1];
          }
          else {
            node.alt += '|link=' + node.link;
          }
          if (typeof node.url != 'undefined') {
            node.alt += '|url=' + node.url;
          }
          node.alt = encodeURIComponent(node.alt);
          return ed.dom.createHTML('img', node);*/
        });
        return;
      });

      // Add a node change handler, selects the button in the UI when an image is selected.
      ed.onNodeChange.add(function(ed, command, node) {
        var nd = node;
        if (!nd) return;
        while (nd.parentNode && nd.className != 'mceItemMediaLibrary')
          nd = nd.parentNode;
        command.setActive('media_library', nd.className == 'mceItemMediaLibrary');
      });
    },

    /**
     * Return information about the plugin as a name/value array.
     */
    getInfo : function() {
      return {
        longname : 'Media Library',
        author : 'Franz Glauber Vanderlinde',
        authorurl : 'http://www.chuva-inc.com',
        infourl : 'http://drupal.org/project/media_library',
        version : "1.0"
      };
    }
  });

  // Register plugin.
  tinymce.PluginManager.add('media_library', tinymce.plugins.MediaLibraryPlugin);
})();
